"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const os_1 = require("os");
const path_1 = require("path");
const docker_command_1 = tslib_1.__importDefault(require("../../docker-command"));
const version_1 = tslib_1.__importDefault(require("../../version"));
const status_1 = tslib_1.__importDefault(require("./status"));
class Start extends docker_command_1.default {
    async run() {
        const { flags } = this.parse(Start);
        const status = await status_1.default.run(['--name', flags.name, '--silent']);
        if (status === 0 /* STARTED */) {
            this.log('Engine is already started');
            return false;
        }
        if (flags.pull) {
            this.spinner.start(`Pulling version ${flags.version}`);
            await this.pull(flags.version);
            this.spinner.stop();
        }
        this.spinner.start('Starting Engine');
        const eventPromise = this.waitForEvent(({ Action, Type, from }) => Type === 'container' &&
            Action === 'start' &&
            from === `mesg/engine:${flags.version}`);
        const network = await this.getOrCreateNetwork({ name: flags.name });
        const tendermintNetwork = await this.getOrCreateNetwork({ name: 'mesg-tendermint' });
        await this.createEngineService(network, tendermintNetwork, {
            name: flags.name,
            version: flags.version,
            colors: flags['log-force-colors'],
            format: flags['log-format'],
            level: flags['log-level'],
            genesisValidatorTx: flags['genesis-validator-tx'],
            genesisTime: flags['genesis-time'],
            chainId: flags['chain-id'],
            persistentPeers: flags.peers || '',
            path: flags.path,
            port: flags.port,
            p2pPort: flags['p2p-port'],
        });
        await eventPromise;
        this.spinner.stop();
        return true;
    }
}
exports.default = Start;
Start.description = 'Start the Engine';
Start.flags = Object.assign(Object.assign({}, docker_command_1.default.flags), { version: command_1.flags.string({
        description: 'Version of the Engine to run',
        required: true,
        default: version_1.default.engine
    }), path: command_1.flags.string({
        description: 'Path to the mesg folder',
        default: path_1.join(os_1.homedir(), '.mesg'),
        required: true,
    }), pull: command_1.flags.boolean({
        description: 'Pull the latest image of the given version',
        default: true,
        allowNo: true
    }), 'log-force-colors': command_1.flags.boolean({
        description: 'Log force colors',
        default: false
    }), 'log-format': command_1.flags.enum({
        description: 'Log format',
        default: 'text',
        options: ['text', 'json']
    }), 'log-level': command_1.flags.enum({
        description: 'Log level',
        default: 'info',
        options: ['debug', 'info', 'warn', 'error', 'fatal', 'panic']
    }), 'genesis-validator-tx': command_1.flags.string({
        description: 'The transaction that add the validators to the genesis',
        required: true,
    }), peers: command_1.flags.string({
        description: 'The list of persistent peers',
    }), 'chain-id': command_1.flags.string({
        description: 'The id of the chain',
        default: 'mesg-chain',
        required: true,
    }), 'genesis-time': command_1.flags.string({
        description: 'The creation time of the genesis',
        default: '2019-01-01T00:00:00Z',
        required: true,
    }), 'p2p-port': command_1.flags.integer({
        description: 'Port to use for p2p interaction',
        default: 26656,
        required: true,
    }) });
