"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const base58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const root_command_1 = tslib_1.__importDefault(require("../../root-command"));
const error_1 = require("../../utils/error");
const compile_1 = tslib_1.__importDefault(require("./compile"));
const create_1 = tslib_1.__importDefault(require("./create"));
const delete_1 = tslib_1.__importDefault(require("./delete"));
const logs_1 = tslib_1.__importDefault(require("./logs"));
class ProcessDev extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.processCreated = false;
        this.instanceCreated = false;
    }
    async run() {
        const { args, flags } = this.parse(ProcessDev);
        this.spinner.start('Starting process');
        this.spinner.status = 'compiling';
        const envToArgs = (flags.env || []).reduce((prev, next) => [...prev, '--env', next], []);
        const definition = await compile_1.default.run([args.PROCESS, '--silent', '--dev', ...envToArgs]);
        this.spinner.status = 'creating process';
        const processHash = await this.createProcess(definition);
        this.spinner.status = 'fetching logs';
        const stream = await logs_1.default.run([base58.encode(processHash)]);
        this.spinner.stop('ready');
        process.once('SIGINT', async () => {
            stream.destroy();
            if (this.processCreated)
                await delete_1.default.run([base58.encode(processHash), '--confirm']);
        });
    }
    async createProcess(definition) {
        try {
            const process = await this.api.process.create(definition);
            if (!process.hash)
                throw new Error('invalid hash');
            this.processCreated = true;
            return process.hash;
        }
        catch (e) {
            if (!error_1.IsAlreadyExistsError.match(e))
                throw e;
            this.warn('process already created');
            return new error_1.IsAlreadyExistsError(e).hash;
        }
    }
}
exports.default = ProcessDev;
ProcessDev.description = 'Run a process in development mode';
ProcessDev.flags = Object.assign(Object.assign(Object.assign({}, root_command_1.default.flags), compile_1.default.flags), create_1.default.flags);
ProcessDev.args = [{
        name: 'PROCESS',
        description: 'Path of the process',
        default: './'
    }];
