"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const base58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const credential_command_1 = require("../../credential-command");
const start_1 = tslib_1.__importDefault(require("./start"));
class ServiceCreate extends credential_command_1.WithCredential {
    async run() {
        const { args, flags } = this.parse(ServiceCreate);
        const credential = await this.getCredential();
        this.spinner.start('Create service');
        const service = JSON.parse(args.DEFINITION);
        const resp = await this.api.service.create(service, credential);
        if (!resp.hash)
            throw new Error('invalid hash');
        this.spinner.stop(base58.encode(resp.hash));
        if (flags.start) {
            this.spinner.start('Starting service');
            const start = await start_1.default.run([base58.encode(resp.hash)]);
            this.spinner.stop(base58.encode(start.hash));
        }
        return { hash: resp.hash };
    }
}
exports.default = ServiceCreate;
ServiceCreate.description = 'Create a service';
ServiceCreate.flags = Object.assign(Object.assign({}, credential_command_1.WithCredential.flags), { start: command_1.flags.boolean({
        description: 'Automatically start the service once created',
        default: false,
    }) });
ServiceCreate.args = [{
        name: 'DEFINITION',
        required: true,
        description: 'Service\'s definition. Use service:compile first to build service definition'
    }];
