"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cli_ux_1 = tslib_1.__importDefault(require("cli-ux"));
const base58 = tslib_1.__importStar(require("mesg-js/lib/util/base58"));
const root_command_1 = tslib_1.__importDefault(require("../../root-command"));
class ServiceList extends root_command_1.default {
    async run() {
        const { flags } = this.parse(ServiceList);
        const [{ services }, { instances }, { ownerships }] = await Promise.all([
            this.api.service.list({}),
            this.api.instance.list({}),
            this.api.ownership.list({})
        ]);
        cli_ux_1.default.table(services || [], {
            hash: { header: 'HASH', get: x => x.hash ? base58.encode(x.hash) : '' },
            sid: { header: 'SID', get: x => x.sid },
            ownerships: {
                header: 'OWNER',
                get: x => (ownerships || [])
                    .filter(y => y.serviceHash && x.hash && y.serviceHash.toString() === x.hash.toString())
                    .map(y => y.owner)
                    .join('\n')
            },
            instances: {
                header: 'INSTANCES',
                get: x => (instances || [])
                    .filter(y => y.serviceHash && x.hash && y.serviceHash.toString() === x.hash.toString())
                    .map(y => y.hash && base58.encode(y.hash))
                    .join('\n')
            },
        }, Object.assign({ printLine: this.log }, flags));
        return instances || [];
    }
}
exports.default = ServiceList;
ServiceList.description = 'List running services';
ServiceList.flags = Object.assign(Object.assign({}, root_command_1.default.flags), cli_ux_1.default.table.flags());
