"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const inquirer_1 = require("inquirer");
const root_command_1 = tslib_1.__importDefault(require("./root-command"));
class WithCredentialPassphrase extends root_command_1.default {
    async getCredentialPassphrase() {
        const { flags } = this.parse();
        if (flags.passphrase)
            return flags.passphrase;
        return cli_ux_1.cli.prompt('Type the passphrase', { type: 'hide' });
    }
}
exports.WithCredentialPassphrase = WithCredentialPassphrase;
WithCredentialPassphrase.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { passphrase: command_1.flags.string({
        description: 'Passphrase of the account'
    }) });
class WithCredential extends WithCredentialPassphrase {
    async getCredential() {
        return {
            username: await this.getCredentialUsername(),
            passphrase: await this.getCredentialPassphrase(),
        };
    }
    async getCredentialUsername() {
        const { flags } = this.parse();
        if (flags.account)
            return flags.account;
        const { accounts } = await this.api.account.list({});
        if (!accounts)
            throw new Error('no account found, please run `mesg-cli account:create ACCOUNT_NAME`');
        const prompt = inquirer_1.createPromptModule({ output: process.stderr });
        const { value } = (await prompt({
            type: 'list',
            name: 'value',
            message: 'Select the account to use',
            default: 'Basic',
            choices: accounts.map((x) => ({
                name: `${x.name} ➜ ${x.address}`,
                value: x.name,
                short: x.name
            }))
        }));
        return value;
    }
}
exports.WithCredential = WithCredential;
WithCredential.flags = Object.assign(Object.assign({}, WithCredentialPassphrase.flags), { account: command_1.flags.string({
        description: 'Name of the account'
    }) });
