"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const node_docker_api_1 = require("node-docker-api");
const debug = require('debug')('docker');
const root_command_1 = tslib_1.__importDefault(require("./root-command"));
class default_1 extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.docker = new node_docker_api_1.Docker(null);
    }
    async listServices(options) {
        return this.docker.service.list({
            filters: { name: [options.name] }
        });
    }
    async waitForEvent(matchFilter) {
        return new Promise(async (resolve) => {
            const events = (await this.docker.events());
            const handler = (buffer) => {
                try {
                    const event = JSON.parse(buffer.toString());
                    if (matchFilter(event)) {
                        events.destroy();
                        return resolve();
                    }
                }
                catch (_a) { }
            };
            events.addListener('data', handler);
        });
    }
    async getOrCreateNetwork(options) {
        const networks = await this.docker.network.list({
            filters: { name: [options.name] }
        });
        if (networks.length > 0) {
            return networks[0];
        }
        return this.docker.network.create({
            CheckDuplicate: true,
            Driver: 'overlay',
            Name: options.name,
            labels: {
                'com.docker.stack.namespace': options.name,
            }
        });
    }
    async createEngineService(network, tendermintNetwork, options) {
        const image = `mesg/engine:${options.version}`;
        if (!fs_1.existsSync(options.path)) {
            fs_1.mkdirSync(options.path);
        }
        return this.docker.service.create({
            Name: options.name,
            Labels: {
                'com.docker.stack.namespace': options.name,
                'com.docker.stack.image': image,
            },
            TaskTemplate: {
                ContainerSpec: {
                    Image: image,
                    Labels: {
                        'com.docker.stack.namespace': options.name
                    },
                    Env: [
                        `MESG_LOG_FORMAT=${options.format}`,
                        `MESG_LOG_LEVEL=${options.level}`,
                        `MESG_LOG_FORCECOLORS=${options.colors}`,
                        `MESG_NAME=${options.name}`,
                        `MESG_TENDERMINT_P2P_PERSISTENTPEERS=${options.persistentPeers}`,
                        `MESG_COSMOS_CHAINID=${options.chainId}`,
                        `MESG_COSMOS_GENESISVALIDATORTX=${options.genesisValidatorTx}`,
                        `MESG_COSMOS_GENESISTIME=${options.genesisTime}`,
                    ],
                    Mounts: [{
                            Source: '/var/run/docker.sock',
                            Target: '/var/run/docker.sock',
                            Type: 'bind',
                        }, {
                            Source: options.path,
                            Target: '/root/.mesg',
                            Type: 'bind',
                        }],
                },
                Networks: [
                    {
                        Target: network.id,
                        Alias: options.name,
                    },
                    {
                        Target: tendermintNetwork.id,
                        Alias: options.name,
                    },
                ]
            },
            EndpointSpec: {
                Ports: [{
                        Protocol: 'tcp',
                        PublishMode: 'ingress',
                        TargetPort: 50052,
                        PublishedPort: options.port,
                    }, {
                        Protocol: 'tcp',
                        PublishMode: 'ingress',
                        TargetPort: 26656,
                        PublishedPort: options.p2pPort,
                    }],
            }
        });
    }
    async pull(tag) {
        const stream = (await this.docker.image.create({}, {
            fromImage: 'mesg/engine',
            tag
        }));
        return new Promise((resolve, reject) => stream
            .on('data', (x) => debug(x.toString())) // For some reason we need to listen to the data to have the end event
            .on('error', reject)
            .on('end', resolve));
    }
}
exports.default = default_1;
default_1.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { name: command_1.flags.string({
        description: 'Name of the docker service running the engine',
        required: true,
        default: 'engine'
    }) });
